% This example shows how to compute IRFs, HDs, and FEVDs in a VAR with 
% data for inflation, unemployment, and interest rates using the VAR 
% Toolbox.  Identification of a demand, a supply, and a monetary policy 
% shock is achieved with a sign restriction procedure.  

% The VAR Toolbox 2.0 is required to run this code. To get the 
% latest version of the toolboxes visit: 
% 
%       https://sites.google.com/site/ambropo/MatlabCodes
% 
% =======================================================================
% Ambrogio Cesa Bianchi, March 2015
% ambrogio.cesabianchi@gmail.com


%% 1. PRELIMINARIES
% =======================================================================
clear all; clear session; close all; clc
warning off all

% Load
[xlsdata, xlstext] = xlsread('SW2001_Data.xlsx','Sheet1');
X = log(xlsdata);
% Define label for plots
dates = xlstext(2:end,1);
vnames = xlstext(1,2:end);
% Define number of variables and of observations
[nobs, nvar] = size(X);


%% VAR ESTIMATION
% =======================================================================
% Set the case for the VARout (0, 1, or 2)
det = 2;
% Set number of nlags
nlags = 4;
% Estimate 
[VAR, VARopt] = VARmodel(X,nlags,det);
% Print at screen and create table
VARopt.vnames = vnames;
[beta, tstat, TABLE] = VARprint(VAR,VARopt);


%% SIGN RESTRICTIONS
% =======================================================================
% Supply shock
R(:,:,1) = [ 1     1    -1    % Inflation
             1     1    -1    % Unemployment
             1     1    -1];  % Fed Funds
% Demand shock
R(:,:,2) = [ 1     1     1    % Inflation
             1     1    -1    % Unemployment
             1     1     1];  % Fed Funds

% Monetary policy shock
R(:,:,3) = [ 1     1    -1    % Inflation
             1     1     1    % Unemployment
             1     1     1];  % Fed Funds

% Set options for IRFs and SR
VARopt.quality = 0;
VARopt.snames = {'Supply Shock';'Demand Shock';'Monetary Policy Shock'};
VARopt.vnames = vnames;

% Run sign restrictions routine
SRout = SR(VAR,R,VARopt);

% Plot IRFs
MED = SRout.IRFmed;
INF = SRout.IRFinf;
SUP = SRout.IRFsup;
SRirplot(MED,VARopt,INF,SUP)

% Plot FEVDs
MED = SRout.FEVDmed;
INF = SRout.FEVDinf;
SUP = SRout.FEVDsup;
SRfevdplot(MED,VARopt,INF,SUP)

% Plot HD
MED = SRout.HDmed;
SRhdplot(MED,VARopt);

