function vd = var_decp_zm( coeffs, L, QQQ, horz)
%
% function vardec = var_dec( varstr, nperiods ) computes the variance 
% decomposition of the forecast errors of the VAR(p) system:
%
%    x(t) = Phi_1*x(t-1) + Phi_2*x(t-2) +...+ Phi_p*x(t-p) + eps(t) (1)
%
% where x(t) is a vector with "K" variables, Phi_i are K x K matrices, 
% E(eps(t))=0 and E(eps(t)*eps(t)') = Sigma, a K x K covariance matrix. 
% Because the error terms are possibly correlated, the ordering of the 
% series matter for this statistic. To orthogonalize the system, I find a 
% new shock eta(t) such that eta(t) = Q eps(t) and E(eta(t)*eta(t)')=I. 
% Note that E(eta(t)*eta(t)') = Q*Sigma*Q' = I, so if E(eta*eta')=I, then 
% the Q's must satisfy inv(Q)*inv(Q') = Sigma. To find such a Q, I use the 
% Cholesky decomposition of Sigma, D*D'=Sigma and set Q=inv(D). Then we 
% write the system as:
%
%    x(t) = Phi_1*x(t-1) + Phi_2*x(t-2) +...+ Phi_p*x(t-p) + D*eta(t)
%
% Note that if the covariance matrix of eps(t) is diagonal, all orderings
% give the same answer--this is important, for instance, when solving a
% model with primitive uncorrelated shocks. To implement this routine, I
% write the system as an augmented VAR(1):
%
%   Y(t) = A*Y(t-1) + C*eta(t)                                      (2)
%
% where   _        _      _                                _       _   _
%        |  x(t)    |    | Phi_1  Phi_2 ... Phi_{p-1} Phi_p |     |  D  |
%        | x(t-1)   |    |   I     0           0        0   |     |  0  |
% Y(t) = | x(t-2)   | A =|   0     I           0        0   | C = |  0  |
%        |  :       |    |   :     :                    :   |     |  :  |
%        | x(t-p+1) |    |   0     0           I        0   |     |  0  |
%         -        -      -                                -       -   -
% 

% INPUTS:
% 1) varstr = a matlab structure with the information of the VAR(p). The
%    elements of varstr are:
%      a) varstr.lags  = p (number of lags in the VAR)
%      b) varstr.Phi_i = matrix on lag(i) values, for i=1,2,...,p
%      c) varstr.QQQ = Contemp Impact Matrix (eg chol(VarCov)')
% 2) nperiods: # of periods ahead to compute variance of forecast errors
% Written by Constantino Hevia, 04/21/2009

% ----------------------------------------------------------------------
% I) Check existence and consistency of input data

nvars = size(QQQ,1);
varstr.Phi_1 = coeffs(1:nvars,1:nvars);
for l = 1:L
    varstr.Phi_2 = coeffs(1:nvars,(l-1)*nvars+1:l*nvars);
end
varstr.lags  = L;
varstr.Sigma = QQQ;

D = QQQ;
C = zeros(nvars*L,nvars);
C(1:nvars,:) = D;

% Create companion matrix A:
A = zeros(nvars*L);

for i=1:L
    A(1:nvars,(i-1)*nvars+1:i*nvars) = varstr.(['Phi_',num2str(i)]);
end
A(nvars+1:end,1:nvars*(L-1))=eye(nvars*(L-1));

% ---------------------------------------------------------------------
% III) Compute variance decomposition of forecast errors due to each
% orthogonalized shock.

% Initialize structure to store variance decomposition of forecast errors:
for i=1:nvars   % For each orthogonalized shock
    vardec.(['shock',num2str(i)]) = zeros(horz,nvars);
end
vardec.stdev = zeros(horz,nvars);

variance = zeros(horz,nvars);   % Temporary array to store variances;

for i = 1:nvars   % Iterate over orthogonalized shocks:
    
    % Matrix that identifies the shock:
    Ws = zeros(nvars); Ws(i,i)=1;
    
    % Initialize variance of forecast error in t=1
    v1 = C*Ws*C'; 
    vari = v1;
    
    % Store just the variance of vari:
    vardec.(['shock',num2str(i)])(1,:) =  diag(vari(1:nvars,1:nvars))';
   
    % Compute recursively variance of t-step-ahead forecast error
    for t = 2:horz
        vari = v1 + A*vari*A';                                              % Update covariance matrix of forecast errors
        vardec.(['shock',num2str(i)])(t,:) = diag(vari(1:nvars,1:nvars))';  % Store just the variances of vari
    end
    
    % Update overall variance of t-step-ahead forecast error:
    variance = variance + vardec.(['shock',num2str(i)]);
end

% Now put this information as a percentage of the forecast error variance
for i=1:nvars        % For each series
    for j=1:nvars    % For each shock
        vardec.(['shock',num2str(j)])(:,i)=100*vardec.(['shock',num2str(j)])(:,i)./variance(:,i);
    end
end
vardec.stdev = sqrt(variance);

vd = zeros(horz,nvars);
for i=1:nvars        % For each series
    for j=1:nvars    % For each shock
        vd(:,j,i)=vardec.(['shock',num2str(j)])(:,i);
    end
end